import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.tree.*;
import java.io.PrintWriter;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.FileWriter;

public class ClassCompiler {
  public static void main(String[] args) throws Exception {
    // Process input file.
    String inputFile = null;
    if (args.length > 0) inputFile = args[0];
    CharStream input;
    if (inputFile != null) {
      input = CharStreams.fromFileName(inputFile);
    } else {
      input = CharStreams.fromStream(System.in);
    }

    // Phase 1: Lexing and parsing.
    GrammarLexer lexer = new GrammarLexer(input);
    CommonTokenStream tokens = new CommonTokenStream(lexer);
    GrammarParser parser = new GrammarParser(tokens);

    ParseTree tree = parser.program();
    System.err.println(tree.toStringTree(parser));

    // Phase 2: Type checking.
    ClassTypeChecker classtypechecker = new ClassTypeChecker();
    classtypechecker.visit(tree);

    // Phase 3: C code gen.
    ClassCodeGen classcodegen = new ClassCodeGen(classtypechecker.scope);
    System.out.println(classcodegen.visit(tree));
  }
}
